﻿// IUtility.cs
// This file defines interfaces for passing utility functions


using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace UnbalancedMatchSim
{

    /// <summary>
    /// A Characteristics object allows access to characteristics of the agents on one side of the market. 
    /// It's meant to wrap an array of Dictionary.
    /// </summary>
    public interface ICharacteristics
    {
        double getCharacteristic(int agentIndex, string attribute);

        void printCharac(StreamWriter sw);
    }

    /// <summary>
    ///  Objects implementing this interface are utility models object that can be used to draw characteristics and preferences for both sides.
    /// </summary>
    public interface IUtility
    {

        
        // Draw the charesteristcs for the entire economy
        ICharacteristics drawChar(int K, bool isMen);

        // Generate a utility draw given charasteristics
        double drawUtil(int agentIndex, int spouseIndex, ICharacteristics characteristicsSpouses, ICharacteristics mySideCharestaristics, bool isMen);

        /// <summary>
        /// Gives the utility for remaining unmatched. 
        /// </summary>
        /// <param name="agentIndex"></param>
        /// <param name="spouseIndex"></param>
        /// <param name="characteristicsSpouses"></param>
        /// <param name="mySideCharestaristics"></param>
        /// <returns></returns>
        double getUnmatchedUtil(int agentIndex, ICharacteristics characteristicsSpouses, ICharacteristics mySideCharestaristics, bool isMen);

        void setCoefficient(string coefficientName, double value);

        void setParam(string paramName, int value);

        void writeCoefficients(StreamWriter sw);

        void writeUtilityModelDescrition(StreamWriter sw);
    }


}
